% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proportional.hazards.modelling.R
\name{ph}
\alias{ph}
\title{Treatment Effects on Changes of Clinical Status Over Time}
\usage{
ph(formula, data, subject, treatment, init.status, nmin = 5)
}
\arguments{
\item{formula}{A formula object, with all of the outcome variables
on the left hand side of a \code{~} operator and the treatment and covariates 
on the right. The outcome variables must be specified through the \code{outcome()} function. 
See \code{?outcome} and Details for further information.}

\item{data}{A data.frame object. The data.frame in which to interpret the
variable names in formula. Must contain the subject ID, the treatment arm, 
the initial status, the examination time, the clinical status measured  
at each examination, and any covariates. 
See Details.}

\item{subject}{A character string. Name of the variable in data which 
identifies multiple rows from the same subject.}

\item{treatment}{A character string. Name of the variable in data that 
corresponds to treatment arm.}

\item{init.status}{A character string. Name of the variable in data that 
corresponds to initial status.}

\item{nmin}{A positive integer object. The minimum number of cases for an event
to be taken into account. Any event with fewer cases than this number will not 
be modelled. The default value is 5.}
}
\value{
A matrix containing the estimated hazard ratio for each level of improvement or            deterioration from the clinical status at enrollment, the estimate of a common hazard 
  ratio for any level of improvement, the estimate of a common hazard ratio for any level   of deterioration, and the estimate of a common hazard ratio summarizing the overall 
  treatment benefit,
  together with the 95\% confidence interval and the two-sided p-value for testing 
  no treatment effect on each of the endpoints.
}
\description{
Different levels of improvement or deterioration from the clinical status 
at randomization are viewed as multiple events. 
Treatment effect on each event is formulated through a Cox
proportional hazards model, stratified by the initial status.  
The covariance matrix of all the estimated treatment effects is estimated 
using the Wei, Lin and Weissfeld (1989) method.
Z-scores are used or combined to test various null hypotheses, including
no treatment effect on a particular event, no treatment effect on any improvement 
or deterioration, and no overall benefits.
}
\details{
The information required for an analysis is 
  \describe{
    \item{Subject ID:}{The variable used to identify subjects.}
    \item{Treatment arm:}{The binary indicator of the treatment arm, 
       1 for the treatment group and 0 for the placebo group.}
    \item{Initial Status:}{The clinical status at randomization. NA if missing.}
    \item{Examination Time:}{The times when the subject is examined for clinical status.
      NA if missing.}
    \item{Clinical Status:}{The clinical status of the subject at each examination.
      NA if missing.}
    \item{Covariates:}{Baseline covariates.}
   }

The input data should be in the long format, i.e., each row pertains to 
one examination of one subject.
The subject ID can be numeric or character.
The treatment arm must be 0/1, with 0 and 1 representing the placebo and the treatment 
arms, respectively. 
The examination time must be non-negative integer.
The clinical status must be integer, with values 1, 2, ..., K, where K is the 
total number of categories.
The larger the status, the worse the clinical outcome.   
The covariates can include categorical variables, for which
all other categories are compared to the first category.
A model without covariates is also allowed.   

The general structure of the formula input is
  \preformatted{
  outcome(examination_time, clinical_status) ~ treatment_arm + covariates 
  }

The left-hand side contains the information about the examination time and 
the corresponding clinical status. It must be
specified through function \code{outcome()}. Specifically, 
\preformatted{outcome(examination_time, clinical_status)}
}
\examples{
data(acttData)
model <- outcome(examination_time, clinical_status) ~ treatment_arm + baseline_severity
ph(formula = model, 
   data = acttData,
   subject = "subject_id",
   treatment = "treatment_arm",
   init.status = "initial_status")
}
\references{
Lin DY, Wang J, Gu Y, Zeng D (2022). 
  Evaluating Treatment Efficacy in Hospitalized Covid-19 Patients. Submitted.

  Wei LJ, Lin DY, Weissfeld L (1989). 
  Regression analysis of multivariate incomplete failure time data by modeling marginal distributions. 
  J Am Stat Ass, 84: 1065-1073.
}
